from typing import Iterable

import numpy as np

def validate_args(fname, args, max_fname_arg_count, compat_args) -> None: ...
def validate_kwargs(fname, kwargs, compat_args) -> None: ...
def validate_args_and_kwargs(
    fname, args, kwargs, max_fname_arg_count, compat_args
) -> None: ...
def validate_bool_kwarg(value, arg_name): ...
def validate_axis_style_args(data, args, kwargs, arg_name, method_name): ...
def validate_fillna_kwargs(value, method, validate_scalar_dict_value: bool = ...): ...
def validate_percentile(q: float | Iterable[float]) -> np.ndarray: ...
